@@ -58,11 +58,50 @@ def _merge_elements( elm_def ):
         for i in base_elm:
             # Everything that's not a list. New key/values overwrite old.
             # If base has a base, will be added and loop will pick it up.
-            if i not in ['paths','labels','shapes'] and i not in elm_def:
+            if i not in ['paths','labels','shapes','anchors'] and i not in elm_def:
                 elm_def[i] = base_elm[i]
     return elm_def
 
 
+#--------------------------------------------------------------------
+def group_elements( drawing, anchors={} ):
+    ''' Combine all elements in a drawing into a single element that can be added to
+        another drawing. Returns an element definition.
+
+        drawing: The drawing object. All elements in the drawing will be combined.
+        anchors: New anchor dictionary to use for the new element.
+    '''
+    new_elm = {'paths' :[],
+               'shapes':[],
+               'labels'  :[],
+               'anchors':{} }
+    for elm in drawing._elm_list:
+        new_elm['paths'].extend( elm.paths )
+        for s in elm.shapes:
+            s = s.copy()
+            if s['shape'] == 'circle':
+                s['center'] = elm.translate( s['center'] - elm.ofst )
+            elif s['shape'] == 'poly':
+                s['xy'] = elm.translate( s['xy'] - elm.ofst )
+            elif s['shape'] == 'arc':
+                s['center'] = elm.translate( s['center'] - elm.ofst )            
+            elif s['shape'] == 'arrow':
+                s['start'] = elm.translate( s['start'] - elm.ofst )
+                s['end']   = elm.translate( s['start'] - elm.ofst )
+            new_elm['shapes'].append( s )
+
+        new_elm['anchors'] = anchors
+
+        for label, loc, align, size in elm.strs:
+            loc = elm.translate( np.array(loc) )
+            new_elm['labels'].append( {'label':label, 'pos':loc, 'align':align, 'size':size} )
+        new_elm['extend'] = False
+        new_elm['drop'] = drawing.here
+    
+    return new_elm
+
+
+
 #--------------------------------------------------------------------
 # Main drawing class
 #--------------------------------------------------------------------
@@ -306,6 +345,7 @@ Labels [Default = no label]:
 Other:
     move_cur : move the cursor after drawing. Default=True.
     color    : matplotlib color for element. e.g. 'red', '#34a4e6', (.8,0,.8)
+    ls       : line style. same as matplotlib: ['-', ':', '--']
     """
         # Flatten element def with base elements
         self.defn   = elm_def.copy()
@@ -313,8 +353,10 @@ Other:
             self.defn = _merge_elements( self.defn )
 
         # Get some parameters
+        self.drawing = drawing
         self.shapes = self.defn.get( 'shapes', [] )
         self.color  = kwargs.get( 'color', self.defn.get('color', drawing.color ) )
+        self.ls     = kwargs.get( 'ls', self.defn.get('ls', '-') )
         totlen      = kwargs.get( 'l', drawing.unit   )
 
         # Determine theta angle of element
@@ -372,7 +414,7 @@ Other:
                 totlen = _distance( self.trans, endpt )
             elif 'tox' in kwargs:
                 # Allow either full coordinate (only keeping x), or just an x value
-                if isinstance( kwargs['tox'], float ):
+                if isinstance( kwargs['tox'], float ) or isinstance( kwargs['tox'], int ):
                     x = float(kwargs['tox'])
                 else:
                     x = kwargs['tox'][0]
@@ -380,7 +422,7 @@ Other:
                 totlen = _distance( self.trans, endpt )
             elif 'toy' in kwargs:
                 # Allow either full coordinate (only keeping y), or just a y value
-                if isinstance( kwargs['toy'], float):
+                if isinstance( kwargs['toy'], float) or isinstance( kwargs['toy'], int):
                     y = kwargs['toy']
                 else:
                     y = kwargs['toy'][1]
@@ -424,15 +466,18 @@ Other:
             rgtlabel = kwargs.get( 'rgtlabel', None )
             lftlabel = kwargs.get( 'lftlabel', None )
             self.lbl_ofst = kwargs.get( 'lblofst', self.defn.get( 'lblofst', drawing.txtofst ) )
-            if label is not None:    self.add_label( label, dfltlbl,  ofst=self.lbl_ofst )
-            if toplabel is not None: self.add_label( toplabel, 'top', ofst=self.lbl_ofst )
-            if botlabel is not None: self.add_label( botlabel, 'bot', ofst=self.lbl_ofst )
-            if rgtlabel is not None: self.add_label( rgtlabel, 'rgt', ofst=self.lbl_ofst )
-            if lftlabel is not None: self.add_label( lftlabel, 'lft', ofst=self.lbl_ofst )
+            if label is not None:    self.add_label( label, dfltlbl,  ofst=self.lbl_ofst, size=drawing.fontsize )
+            if toplabel is not None: self.add_label( toplabel, 'top', ofst=self.lbl_ofst, size=drawing.fontsize )
+            if botlabel is not None: self.add_label( botlabel, 'bot', ofst=self.lbl_ofst, size=drawing.fontsize )
+            if rgtlabel is not None: self.add_label( rgtlabel, 'rgt', ofst=self.lbl_ofst, size=drawing.fontsize )
+            if lftlabel is not None: self.add_label( lftlabel, 'lft', ofst=self.lbl_ofst, size=drawing.fontsize )
 
         txtlist = self.defn.get( 'labels', [] )
         for txtlbl in txtlist:
-            self.strs.append( ( (txtlbl[0], np.array(txtlbl[1])-self.ofst, ('center', 'center') ) ) )
+            align = txtlbl.get('align',('center','center') )
+            size  = txtlbl.get('size', drawing.fontsize )
+            pos   = txtlbl.get('pos', [0,0] )
+            self.strs.append( ( (txtlbl['label'], np.array(pos)-self.ofst, align, size ) ) )
 
         self.z = kwargs.get( 'zoom', 1 )
 
@@ -456,6 +501,8 @@ Other:
             setattr(self, 'center', (self.paths[0][0] + self.paths[0][-1])/2 )
         if 'anchors' in self.defn:
             for aname, apoint in self.defn['anchors'].items():
+                if getattr( self, aname, None) is not None: # Try not to clobber element parameter names!
+                    aname = 'anchor_' + aname
                 setattr(self, aname, self.translate(np.array(apoint)-self.ofst))
 
         # Translated end point for new drawing position
@@ -510,13 +557,14 @@ Other:
         return xmin, ymin, xmax, ymax
 
 
-    def add_label( self, label, loc='top', ofst=None, align=None ):
+    def add_label( self, label, loc='top', ofst=None, align=None, size=None ):
         """ Add a label at the appropriate position
             label: text label to add
             loc: location for the label ['top', 'bot', 'lft', 'rgt'].
             txtofst: unit offset between element bounding box and label.
             align: alignment tuple for (horizontal, vertical):
                    (['center', 'left', 'right'], ['center', 'top', 'bottom'])
+            size: font size
         """
         if isinstance(ofst,list) and loc is not 'center':
             raise TypeError( 'Offset must not be list for loc=%s'%loc )
@@ -571,6 +619,9 @@ Other:
         ymax = self.ymax
         ymin = self.ymin
 
+        if size==None:
+            size=self.drawing.fontsize
+
         if self.flip:
             ymax, ymin = ymin, ymax
             if loc=='top' or loc=='bot':
@@ -586,27 +637,27 @@ Other:
                 for i, lbltxt in enumerate(label):
                     xdiv = (xmax-xmin)/(len(label)+1)
                     lbl_pt = [ xmin+xdiv*(i+1), ymax+ofst ]
-                    self.strs.append( (label[i], lbl_pt, align) )
+                    self.strs.append( (label[i], lbl_pt, align, size) )
             elif loc=='bot':
                 for i, lbltxt in enumerate(label):
                     xdiv = (xmax-xmin)/(len(label)+1)
                     lbl_pt = [ xmin+xdiv*(i+1), ymin-ofst ]
-                    self.strs.append( (label[i], lbl_pt, align) )
+                    self.strs.append( (label[i], lbl_pt, align, size) )
             elif loc=='lft':
                 for i, lbltxt in enumerate(label):
                     ydiv = (ymax-ymin)/(len(label)+1)
                     lbl_pt = [ xmin-ofst, ymin+ydiv*(i+1) ]
-                    self.strs.append( (label[i], lbl_pt, align) )
+                    self.strs.append( (label[i], lbl_pt, align, size) )
             elif loc=='rgt':
                 for i, lbltxt in enumerate(label):
                     ydiv = (ymax-ymin)/(len(label)+1)
                     lbl_pt = [ xmax+ofst, ymin+ydiv*(i+1) ]
-                    self.strs.append( (label[i], lbl_pt, align) )
+                    self.strs.append( (label[i], lbl_pt, align, size) )
             elif loc=='center':
                 for i, lbltxt in enumerate(label):
                     xdiv = (xmax-xmin)/(len(label)+1)
                     lbl_pt = [ xmin+xdiv*(i+1), ofst ]
-                    self.strs.append( (label[i], lbl_pt, align) )
+                    self.strs.append( (label[i], lbl_pt, align, size) )
         elif isinstance( label, str ):
             # Place in center
             if loc=='top':
@@ -622,7 +673,7 @@ Other:
                     lbl_pt = [(xmax+xmin)/2+ofst[0], (ymax+ymin)/2+ofst[1]]
                 else:
                     lbl_pt = [(xmax+xmin)/2, (ymax+ymin)/2+ofst]
-            self.strs.append( (label, lbl_pt, align) )
+            self.strs.append( (label, lbl_pt, align, size) )
 
 
     def translate(self, pt, doflip=True):
@@ -646,7 +697,7 @@ Other:
         """
         for path in self.paths:
             ax.plot( path[:,0], path[:,1], color=self.color, lw=1,
-                     solid_capstyle='round' )
+                     solid_capstyle='round', ls=self.ls )
 
         for s in self.shapes:
             if s.get('shape') == 'circle':
@@ -718,9 +769,10 @@ Other:
                          head_width=hw, head_length=hl,
                          length_includes_head=True, color=self.color)
 
-        for label, loc, align in self.strs:
+        for label, loc, align, size in self.strs:
             loc = self.translate( np.array(loc) )
             ha = align[0]
             va = align[1]
             ax.text( loc[0], loc[1], label, transform=ax.transData,
-                     horizontalalignment=ha, verticalalignment=va )
+                     horizontalalignment=ha, verticalalignment=va,
+                     fontsize=size )
